/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (C) 2000 Jeffrey Van Baalen.  All rights reserved.
 *
 * See the file "DisplayAST-LICENSE" for license information.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* This is a simple utility that uses Java's JTree class to provide a
   rudimentary hierarchical display of an AST built by a parser that is 
   generated by SableCC. Essentially all of the code below must be modified
   for you're specific AST classes. The only small complication is that
   token nodes are handled differently from nonterminal nodes (see below).
*/

import java.util.*;
import html.analysis.*;
import html.node.*;
import javax.swing.tree.*;

public class DisplayAST extends DepthFirstAdapter
{
    private Hashtable table = new Hashtable();
    
    // Do not modify the following method.

        public void defaultIn(Node node) {
	table.put(node,new DefaultMutableTreeNode(node.getClass().getName()));
	}

	// Modify all the rest of these methods for you're AST classes. The structure
	// of each out method is very uniform and could probably be generated automatically
	// but do it by hand for now.

    public void outADoc(ADoc node) {
	DefaultMutableTreeNode root = (DefaultMutableTreeNode) table.get(node);
	root.add((DefaultMutableTreeNode)table.get(node.getBhtml()));
	root.add((DefaultMutableTreeNode)table.get(node.getHd()));
	root.add((DefaultMutableTreeNode)table.get(node.getBdy()));
	root.add((DefaultMutableTreeNode)table.get(node.getEhtml()));
	// Bring of the display window (this might not be the best place to do this.
	new DisplayTree(root);
    }

    public void outAHd(AHd node) {
	DefaultMutableTreeNode dnode = (DefaultMutableTreeNode)table.get(node);
	dnode.add((DefaultMutableTreeNode)table.get(node.getBhead()));
	dnode.add((DefaultMutableTreeNode)table.get(node.getBtitle()));
	ListIterator itr = node.getBlock().listIterator();
	while (itr.hasNext()) dnode.add((DefaultMutableTreeNode)table.get(itr.next()));
	dnode.add((DefaultMutableTreeNode)table.get(node.getEtitle()));
	dnode.add((DefaultMutableTreeNode)table.get(node.getEhead()));
    }

    public void outABdy(ABdy node) {
	DefaultMutableTreeNode dnode = (DefaultMutableTreeNode)table.get(node);
	dnode.add((DefaultMutableTreeNode)table.get(node.getBbody()));
	ListIterator itr = node.getBlock().listIterator();
	while (itr.hasNext()) dnode.add((DefaultMutableTreeNode)table.get(itr.next()));
	dnode.add((DefaultMutableTreeNode)table.get(node.getEbody()));
    }

    public void outANormalBlock(ANormalBlock node) {
	DefaultMutableTreeNode dnode = (DefaultMutableTreeNode)table.get(node);
	dnode.add((DefaultMutableTreeNode)table.get(node.getOpentag()));
	ListIterator itr = node.getBlock().listIterator();
	while (itr.hasNext()) dnode.add((DefaultMutableTreeNode)table.get(itr.next()));
	dnode.add((DefaultMutableTreeNode)table.get(node.getClosetag()));
    }

    public void outAMixedBlock(AMixedBlock node) {
	DefaultMutableTreeNode dnode = (DefaultMutableTreeNode)table.get(node);
	// Token nodes are handled as below.
	dnode.add(new DefaultMutableTreeNode(node.getWord()));
    }

    public void outATableBlock(ATableBlock node) {
	DefaultMutableTreeNode dnode = (DefaultMutableTreeNode)table.get(node);
	dnode.add((DefaultMutableTreeNode)table.get(node.getTbl()));
    }

    public void outATbl(ATbl node) {
	DefaultMutableTreeNode dnode = (DefaultMutableTreeNode)table.get(node);
	dnode.add(new DefaultMutableTreeNode(node.getBtable()));
	ListIterator itr = node.getBlock().listIterator();
	while (itr.hasNext()) dnode.add((DefaultMutableTreeNode)table.get(itr.next()));
	dnode.add(new DefaultMutableTreeNode(node.getEtable()));
    }

    public void outAOpentag(AOpentag node) {
	DefaultMutableTreeNode dnode = (DefaultMutableTreeNode)table.get(node);
	dnode.add(new DefaultMutableTreeNode(node.getTag()));
    }

    public void outAClosetag(AClosetag node) {
	DefaultMutableTreeNode dnode = (DefaultMutableTreeNode)table.get(node);
	dnode.add(new DefaultMutableTreeNode(node.getTag()));
    }
	
}
