/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This file is part of MiniBasic.                                 *
 * See the file "MiniBasic-LICENSE" for Copyright information and  *
 * the terms and conditions for copying, distribution and          *
 * modification of MiniBasic.                                      *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

import minibasic.node.*;
import minibasic.lexer.*;
import minibasic.parser.*;

import java.io.*;

public class Main
{
    public static void main(String[] arguments)
    {
        if(arguments.length != 1)
        {
            System.out.println("usage:");
            System.out.println("  java minibasic.Main filename");
            System.exit(1);
        }

        try
        {
            Lexer lexer = new Lexer(
                new PushbackReader(
                new BufferedReader(
                new FileReader(arguments[0])), 1024));

            Parser parser = new Parser(lexer);

            Node ast = parser.parse();

            ast.apply(new Interpreter());
        }
        catch(Exception e)
        {
            System.out.println(e);
        }
    }
}
