/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This file is part of SimpleC.                                   *
 * See the file "SimpleC-LICENSE" for Copyright information and    *
 * the terms and conditions for copying, distribution and          *
 * modification of SimpleC.                                        *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

import ca.mcgill.sable.simple.node.*;
import ca.mcgill.sable.simple.lexer.*;
import ca.mcgill.sable.simple.parser.*;
import java.io.*;

public class Main
{
    public static void main(String[] arguments)
    {
        if(arguments.length != 1)
        {
            System.out.println("usage:");
            System.out.println("  java Main filename");
            System.exit(1);
        }

        compile("", arguments[0]);
    }

    private static void compile(String dir, String name)
    {
        File f;

        if(dir.equals(""))
        {
            f = new File(name);
        }
        else
        {
            f = new File(dir, name);
        }

        if(f.isFile())
        {
            compile(f);
        }
        else
        {
            System.out.println("Error: " + f.getAbsolutePath() + " does not exist.");
        }
    }

    private static void compile(File file)
    {
        try
        {
            System.out.println();
            System.out.println("***************************************");
            System.out.println(file.getAbsolutePath());

            Lexer lexer = new Lexer(
                new PushbackReader(
                new BufferedReader(
                new FileReader(file)), 1024));

            Parser parser = new Parser(lexer);

            Start ast = parser.parse();

            ast.apply(new ModifyAST());

            Declarations declarations = new Declarations(ast);
            PointsTo pto = new PointsTo(ast, declarations);

            System.out.println(pto);
        }
        catch(Exception e)
        {
            e.printStackTrace(System.out);
        }
    }
}
