/**
 * Copyright (c) 2000, Fidel Viegas (viegasfh@hotmail.com).
 * All rights reserved.
 *
 * Please read the LICENSE file for license information.
 *
 * This is a driver program for the smallPascal compiler. This file is similar
 * to the one provided in the calculator example found in the thesis
 * report of SableCC.
 */

package SmallPascal.compiler;

import SmallPascal.lexer.*;
import SmallPascal.parser.*;
import SmallPascal.node.*;
import SmallPascal.semantic.*;
import SmallPascal.code.*;

import java.io.*;

public class Main {
  public static void main(String[] args) {
    System.out.println("SmallPascal version 1.0");
    System.out.println("Copyright (c) 2000, Fidel Viegas (viegasfh@hotmail.com)");
    System.out.println("All rights reserved.");
    System.out.println();
    System.out.println("This software comes with ABSOLUTELY NO WARRANTY.");
    System.out.println("Please read the LICENSE file for license information.");
    System.out.println();

    // check the number of arguments
    if (args.length != 1 || !args[0].endsWith(".pas")) {
      System.out.println("Usage: ");
      System.out.println("   java SmallPascal.compiler.Main filename.pas");
      System.exit(0);
    } 

    try {
      Lexer lexer = new Lexer( 
                      new PushbackReader(
                        new BufferedReader(
                          new FileReader(args[0])), 1024));

      // create an instance of the parser
      Parser parser = new Parser(lexer);

      // parse the program and assign the generated AST to node root
      System.out.println("Parsing " + args[0] + "...");
      Node root = parser.parse();
      
      // process semantic analysis
      System.out.println("Checking identifiers...");
      root.apply(new SemanticAnalyser());

      // process code generation
      System.out.println("Generating " +
              args[0].substring(0, args[0].length() - 4) + ".j");
      root.apply(new CodeGenerator(args[0]));

      // finished compilation
      System.out.println("done.");
    }
    catch(Exception exc) {
      System.out.println(exc);  // output the exception                            
    }
  }
}
